#!/bin/bash
set -ex

cd /etc/vmware/nsx
cp -f /etc/antrea/nsx-cert/tls.crt host-cert.pem
cp -f /etc/antrea/nsx-cert/tls.key host-privkey.pem
if [ -f /root/.local/pipx/venvs/yq/bin/activate ]; then
  source /root/.local/pipx/venvs/yq/bin/activate
fi
while :
do
  # https://github.com/kislyuk/yq 2.14.0 requires double quotes for "rest-api"
  # https://github.com/mikefarah/yq 4.28.2 doesn't have above requirement
  restproxy_len="$(cat /etc/antrea/bootstrap.conf | yq -r '.proxyEndpoints."rest-api"|length')"
  managers_len="$(cat /etc/antrea/bootstrap.conf | yq -r '.NSXManagers|length')"
  if [ "${restproxy_len}" -gt 0 ] || [ "${managers_len}" -gt 0 ]; then
    break
  else
    echo "No NSX Managers or Proxies found, retry after 5s"
    sleep 5
  fi
done
manager=""
echo "NSX manager proxies count: $restproxy_len"
for i in $(seq 0 $((restproxy_len-1))); do
        try_manager="$(cat /etc/antrea/bootstrap.conf | yq -r .proxyEndpoints.\"rest-api\"[$i])"
        # Add square brackets for IPv6 address
        if [[ ${try_manager} =~ ^[^\[].*:.*:.*$ ]]
        then
                try_manager="[${try_manager}]"
        fi
        echo "Trying to connect to proxy $try_manager"
        if curl -s -k --connect-timeout 15 https://$try_manager; then
                manager="${try_manager}"
                echo "Connected to proxy $try_manager"
                break
        else
                echo "Failed to connect to proxy $try_manager"
        fi
done
if [ "${manager}" == "" ]; then
        echo "NSX managers count: $managers_len"
        for i in $(seq 0 $((managers_len-1))); do
                try_manager="$(cat /etc/antrea/bootstrap.conf | yq -r .NSXManagers[$i])"
                # Add square brackets for IPv6 address
                if [[ ${try_manager} =~ ^[^\[].*:.*:.*$ ]]
                then
                        try_manager="[${try_manager}]"
                fi
                echo "Trying to connect to $try_manager"
                if curl -s -k --connect-timeout 15 https://$try_manager; then
                        manager="${try_manager}"
                        echo "Connected to $try_manager"
                        break
                else
                        echo "Failed to connect to $try_manager"
                fi
        done
        if [ "${manager}" == "" ]; then
                echo "no available nsx managers"
                exit 1
        fi
fi

rpcproxy_len="$(cat /etc/antrea/bootstrap.conf | yq -r '.proxyEndpoints."nsx-rpc-fwd-proxy"|length')"
echo "NSX rpc proxies count: $rpcproxy_len"
if [ "${rpcproxy_len}" -gt 0 ]; then
  cat >http-proxy-config.xml <<EOF
<?xml version="1.0" encoding="utf-8"?>
<config>
  <httpProxiesList>
EOF
  for i in $(seq 0 $((rpcproxy_len-1))); do
    try_proxy="$(cat /etc/antrea/bootstrap.conf | yq -r .proxyEndpoints.\"nsx-rpc-fwd-proxy\"[$i])"
    if [[ ${try_proxy} != tcp://* ]]; then
      try_proxy=tcp://${try_proxy}
    fi
    cat >>http-proxy-config.xml <<EOF
    <httpProxy>
      <endpoint>${try_proxy}</endpoint>
    </httpProxy>
EOF
  done
  cat >>http-proxy-config.xml <<EOF
  </httpProxiesList>
</config>
EOF
fi

clusterID="$(cat /etc/antrea/cluster-id.conf | yq -r .clusterID)"
while [ "${clusterID}" == "" ]
do
  echo "tn-proxy init waits for clusterID"
  sleep 15
  clusterID="$(cat /etc/antrea/cluster-id.conf | yq -r .clusterID)"
done

mkdir -p /var/run/vmware/nsx-proxy/

curl -k --cert host-cert.pem --key host-privkey.pem "https://${manager}/api/v1/cluster" > cluster.json
curl -k --cert host-cert.pem --key host-privkey.pem "https://${manager}/api/v1/cluster/nodes" > nodes.json

manager_count="$(cat cluster.json | jq ".nodes | length")"

cat >appliance-info.xml <<EOF
<?xml version="1.0" encoding="utf-8"?>
<appliance-info>
        <appliance-proxies>
EOF

for I in $(seq 0 $((manager_count-1)) ); do
  aph_ep_id="$(cat nodes.json | jq -r ".results | map(select(.manager_role))[${I}].manager_role.appliance_connection_info.service_endpoint_uuid")"
  aph_ip="$(cat nodes.json | jq -r ".results | map(select(.manager_role))[${I}].manager_role.appliance_connection_info.ip_address")"
  aph_ipv6="$(cat nodes.json | jq -r ".results | map(select(.manager_role))[${I}].manager_role.appliance_connection_info.ipv6_address")"
  aph_port="$(cat nodes.json | jq -r ".results | map(select(.manager_role))[${I}].manager_role.appliance_connection_info.port")"
  aph_cert="$(cat nodes.json | jq -r ".results | map(select(.manager_role))[${I}].manager_role.appliance_connection_info.certificate")"
  aph_cert="$(echo -n -e "${aph_cert}")"
  aph_entity_id="$(cat nodes.json | jq -r ".results | map(select(.manager_role))[${I}].id")"
  cat >>appliance-info.xml <<EOF
                <appliance-proxy>
                        <uuid>${aph_ep_id}</uuid>
                        <ip>${aph_ip}</ip>
EOF
  if [ "$aph_ipv6" != "null" ]; then
    # aph_ipv6 gets a "null" value when NSX is IPv4-only.
    cat >>appliance-info.xml <<EOF
                          <ipv6>${aph_ipv6}</ipv6>
EOF
  fi
  cat >>appliance-info.xml <<EOF
                        <fqdn/>
                        <port>${aph_port}</port>
                        <ssl-enabled>true</ssl-enabled>
                        <certificate>${aph_cert}</certificate>
                        <entities>
                                <entity>
                                        <type>MP</type>
                                        <uuid>${aph_entity_id}</uuid>
                                </entity>
                        </entities>
                </appliance-proxy>
EOF
done

cat >>appliance-info.xml <<EOF
        </appliance-proxies>
</appliance-info>
EOF

cat >controller-info.xml <<EOF
<?xml version="1.0" encoding="utf-8"?>
<config>
        <connectionList>
EOF

for I in $(seq 0 $((manager_count-1)) ); do
  ccp_entity_id="$(cat nodes.json | jq -r ".results | map(select(.controller_role))[${I}].id")"
  ccp_ip="$(cat nodes.json | jq -r ".results | map(select(.controller_role))[${I}].controller_role.control_plane_listen_addr.ip_address")"
  ccp_ipv6="$(cat nodes.json | jq -r ".results | map(select(.controller_role))[${I}].controller_role.control_plane_listen_addr.ipv6_address")"
  ccp_port="$(cat nodes.json | jq -r ".results | map(select(.controller_role))[${I}].controller_role.control_plane_listen_addr.port")"
  ccp_cert="$(cat nodes.json | jq -r ".results | map(select(.controller_role))[${I}].controller_role.control_plane_listen_addr.certificate")"
  ccp_cert="$(echo -n -e "${ccp_cert}")"
  cat >>controller-info.xml <<EOF
                <connection>
                        <server>${ccp_ip}</server>
                        <port>${ccp_port}</port>
EOF
  if [ "$ccp_ipv6" != "null" ]; then
    # ccp_ipv6 gets a "null" value when NSX is IPv4-only.
    cat >>controller-info.xml <<EOF
                        <serverV6>${ccp_ipv6}</severV6>
                        <portV6>${ccp_port}</portV6>
EOF
  fi
  cat >>controller-info.xml <<EOF
                        <sslEnabled>true</sslEnabled>
                        <version>1234</version>
                        <uuid>${ccp_entity_id}</uuid>
                        <pemKey>${ccp_cert}</pemKey>      
                </connection>
EOF
done

cat >>controller-info.xml <<EOF
        </connectionList>
        <transportNode>
                <UUID>${clusterID}</UUID>
        </transportNode>
</config>
EOF

cat >host-cfg.xml <<EOF
<?xml version="1.0" ?>
<config>
    <uuid>${clusterID}</uuid>
    <hostPrivateKeyFile>/etc/vmware/nsx/host-privkey.pem</hostPrivateKeyFile>
    <hostCertificateFile>/etc/vmware/nsx/host-cert.pem</hostCertificateFile>
</config>
EOF

cat >nsx-proxy.xml <<EOF
<!-- Configuration file for NSX Proxy Agent -->
<config>
<log>
   <!-- library log level - 0:off 1:fatal 2:error 3:warn 4:info 5:debug 6:trace -->
   <level>4</level>
   <proxy>4</proxy>
   <metrics>4</metrics>
   <net>4</net>
   <rpc>4</rpc>
   <nestdb>4</nestdb>
   <core>4</core>

   <dumpConfigUpdateToLog>true</dumpConfigUpdateToLog>
   <outputToConsole>true</outputToConsole>
</log>
<metrics>
   <!-- see nsx/common/metrics for details -->
   <root_namespace>/nsx/proxy</root_namespace>
   <report_interval_sec>300</report_interval_sec>
   <publishers>
      <rpc/>
   </publishers>
</metrics>
<profiler>
   <prof_prefix>/var/log/vmware/nsx-proxy/nsx-proxy</prof_prefix>
   <!-- seconds -->
   <cpu_prof_interval>0</cpu_prof_interval>
   <mem_prof_interval>900</mem_prof_interval>
   <mem_stats_interval>900</mem_stats_interval>
   <mem_release_interval>86400</mem_release_interval>
</profiler>
<proxy>
   <configManager>
      <listeningMode>false</listeningMode>
      <listeningEndpoint/>
      <configFile>/etc/vmware/nsx/controller-info.xml</configFile>
      <hostPrivateKeyFile>/etc/vmware/nsx/host-privkey.pem</hostPrivateKeyFile>
      <hostCertificateFile>/etc/vmware/nsx/host-cert.pem</hostCertificateFile>
      <ccpConnectionConfig>
         <!-- all values in seconds -->
         <keepAliveTimeout />
         <connectTimeout>10</connectTimeout>
         <!-- set both min and max to 0 to disable -->
         <minConnectBackoff>1</minConnectBackoff>
         <maxConnectBackoff>5</maxConnectBackoff>
      </ccpConnectionConfig>
      <rpcCliListeningEndpoint/>
      <ccpConnectionAlarmConfig>
         <ccpConnectionAlarmEnabled>true</ccpConnectionAlarmEnabled>
      </ccpConnectionAlarmConfig>
   </configManager>
   <ssl>
      <!-- enable TLS1.1, TLS1.2 by default -->
      <protocols>tls1.1,tls1.2</protocols>
   </ssl>
   <nsxagentLink>
      <path>/var/run/vmware/nsx-agent/nsxa_link</path>
   </nsxagentLink>
   <endpoints>
      <nestdb/>
      <tcpForwarding>tcp://127.0.0.1:2081</tcpForwarding>
   </endpoints>
</proxy>
<mock>
   <!-- Count of user dispatchers & internal dispatchers each. So total threads will be 2x this value. -->
   <numDispatchers>10</numDispatchers>
</mock>
</config>
EOF

cat >openssl-proxy.cnf <<EOF
[req]
default_bits            = 2048
distinguished_name      = req_distinguished_name
#Don't encrypt the key
encrypt_key             = no
prompt                  = no
string_mask             = nombstr

[ req_distinguished_name ]
countryName             = US
stateOrProvinceName     = California
localityName            = Palo Alto
0.organizationName      = VMware, Inc.
emailAddress            = ssl-certificates@vmware.com
commonName              = VMware-NSX-Host
EOF

exec /usr/bin/nsx-proxy -c /etc/vmware/nsx/nsx-proxy.xml
